from importlib import import_module
from thehsi import CallData, Kernel
from os import listdir, mkdir
from os.path import exists

def is_module_valid(module):
    valid_chars: str = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_"
    return not bool(
        len(
            list(
                filter(
                    lambda x : x,
                    [not i in valid_chars for i in module]
                )
            )
        )
    )

def load_callers(application_path: str):
    application_path: str = str(
        application_path.replace(
            "\\",
            "/"
        ) # Windows Support
    )

    if not exists(
        f"{application_path}/"
    ):
        print("[ERROR]: The Path you entered does not exist!")
        exit()

    if not exists(
        f"{application_path}/applications/"
    ):
        print("[INFO]: No 'application' folder found! Creating...")
        mkdir(f"{application_path}/applications/")
    
    applications: list[str] = []
    applications: list[str] = list(
        listdir(
            f"{application_path}/applications/"
        )
    )

    callers: dict[dict] = {}

    for application in applications:
        if not is_module_valid(application):
            print(f"[WARNING]: Application '{application}' could not be loaded!\n  The application has an invalid name\n")
            continue
        
        if not exists(
            f'{application_path}/applications/{application}/__hmgr_caller__.py'
        ): continue

        application_caller = import_module(
            f'applications.{application}.__hmgr_caller__'
        )

        kernel: Kernel = application_caller.kernel
        if kernel.app_id != application:
            print(f"[WARNING]: Application '{application}' could not be loaded!\n  The application folder name must match the id\n")
            continue
        application_callers: dict = kernel.__callers__
        for caller_key in application_callers.keys():
            caller = application_callers[caller_key]
            callers[
                f"{caller['application_id']}:{caller_key}"
            ] = caller

    return callers

def call(callers: dict, caller_key: str, args: list = []):
    if not caller_key in callers:
        print(f"[ERROR]: Cannot Call '{caller_key}'\n  No such call\n")
        return
    call_data: CallData = CallData()
    caller: dict = callers[caller_key]
    func: callable = caller['function']
    return func(call_data, args)


from setuptools import setup

version = '0.2.2'

setup(
    name='thehsi',
    version=version,
    packages=['thehsi'],
    license='MIT',
    description='A Python Package for coding a custom TheHSI Hub integrated Application',
    author='TheHSI',
    author_email='central.thehsi@gmail.com',
    url='https://github.com/hstoreinteractive/thehsi-python',
    download_url=f'https://github.com/hstoreinteractive/thehsi-python/archive/refs/tags/v{version}.tar.gz',
    keywords=['thehsi', 'application'],
    install_requires=[
        'requests'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.12',
    ],
)



class CallData:
    remote = ('127.0.0.1', 0)

class Kernel:
    def __init__(self, app_name: str, app_id: str):
        self.__callers__: dict = {}
        self.app_name: str = app_name
        self.app_id: str = app_id
    
    def callable(self, path: str, name: str):
        globals()['__kernel__'] = self
        def callable_function(func: callable):
            def wrapper(call_data: CallData, args: list):
                return func(call_data, args)
            self.__callers__[f'{path.removeprefix("/")}/{name}'] = {
                'name': name,
                'path': path.removeprefix('/'),
                'function': wrapper,
                'application_id': self.app_id,
                'application_name': self.app_name
            }
            return wrapper
        return callable_function